import torch
import torch.nn as nn


class ResBlock(nn.Module):
    def __init__(self, in_channels, out_channels, kernel_size, stride=1, padding=0):
        super(ResBlock, self).__init__()
        self.conv1 = nn.Conv2d(in_channels, out_channels, kernel_size, stride=stride, padding=padding)
        self.bn1 = nn.BatchNorm2d(out_channels)
        self.conv2 = nn.Conv2d(out_channels, out_channels, kernel_size, stride=stride, padding=padding)
        self.bn2 = nn.BatchNorm2d(out_channels)
        self.relu = nn.ReLU()

    def forward(self, s):
        identity = s

        s = self.conv1(s)
        s = self.bn1(s)
        s = self.relu(s)

        s = self.conv2(s)
        s = self.bn2(s)
        s += identity
        s = self.relu(s)

        return s


class ResNet(nn.Module):
    def __init__(self, channels, kernel_size, stride=1, padding=0, layers=1):
        super(ResNet, self).__init__()
        self.block = ResBlock(channels, channels, kernel_size, stride=stride, padding=padding)
        self.net = nn.Sequential(*([self.block]*layers))

    def forward(self, s):
        return self.net(s)
